@@ -1,9 +1,9 @@
 /* Create a hard link relative to open directories.
-   Copyright (C) 2009-2013 Free Software Foundation, Inc.
+   Copyright (C) 2009-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* written by Eric Blake */
 
@@ -29,21 +29,11 @@
 
 #include "areadlink.h"
 #include "dirname.h"
+#include "eloop-threshold.h"
 #include "filenamecat.h"
 #include "openat-priv.h"
 
-#if HAVE_SYS_PARAM_H
-# include <sys/param.h>
-#endif
-#ifndef MAXSYMLINKS
-# ifdef SYMLOOP_MAX
-#  define MAXSYMLINKS SYMLOOP_MAX
-# else
-#  define MAXSYMLINKS 20
-# endif
-#endif
-
-#if !HAVE_LINKAT
+#if !HAVE_LINKAT || LINKAT_SYMLINK_NOTSUP
 
 /* Create a link.  If FILE1 is a symlink, either create a hardlink to
    that symlink, or fake it by creating an identical symlink.  */
@@ -74,10 +64,8 @@ link_immediate (char const *file1, char const *file2)
           if (st1.st_dev == st2.st_dev)
             {
               int result = symlink (target, file2);
-              int saved_errno = errno;
               free (target);
               free (dir);
-              errno = saved_errno;
               return result;
             }
           free (target);
@@ -105,7 +93,7 @@ link_follow (char const *file1, char const *file2)
   char *name = (char *) file1;
   char *target;
   int result;
-  int i = MAXSYMLINKS;
+  int i = __eloop_threshold ();
 
   /* Using realpath or canonicalize_file_name is too heavy-handed: we
      don't need an absolute name, and we don't need to resolve
@@ -147,20 +135,12 @@ link_follow (char const *file1, char const *file2)
   if (!target && errno != EINVAL)
     {
       if (name != file1)
-        {
-          int saved_errno = errno;
-          free (name);
-          errno = saved_errno;
-        }
+        free (name);
       return -1;
     }
   result = link (name, file2);
   if (name != file1)
-    {
-      int saved_errno = errno;
-      free (name);
-      errno = saved_errno;
-    }
+    free (name);
   return result;
 }
 # endif /* 0 < LINK_FOLLOWS_SYMLINKS */
@@ -195,6 +175,10 @@ solaris_optimized_link_follow (char const *file1, char const *file2)
 
 # endif
 
+#endif /* !HAVE_LINKAT || LINKAT_SYMLINK_NOTSUP  */
+
+#if !HAVE_LINKAT
+
 /* Create a link to FILE1, in the directory open on descriptor FD1, to FILE2,
    in the directory open on descriptor FD2.  If FILE1 is a symlink, FLAG
    controls whether to dereference FILE1 first.  If possible, do it without
@@ -227,7 +211,7 @@ linkat_follow (int fd1, char const *file1, int fd2, char const *file2)
   char *name = (char *) file1;
   char *target;
   int result;
-  int i = MAXSYMLINKS;
+  int i = __eloop_threshold ();
 
   /* There is no realpathat.  */
   while (i-- && (target = areadlinkat (fd1, name)))
@@ -267,20 +251,12 @@ linkat_follow (int fd1, char const *file1, int fd2, char const *file2)
   if (!target && errno != EINVAL)
     {
       if (name != file1)
-        {
-          int saved_errno = errno;
-          free (name);
-          errno = saved_errno;
-        }
+        free (name);
       return -1;
     }
   result = linkat (fd1, name, fd2, file2, 0);
   if (name != file1)
-    {
-      int saved_errno = errno;
-      free (name);
-      errno = saved_errno;
-    }
+    free (name);
   return result;
 }
 
@@ -321,7 +297,17 @@ rpl_linkat (int fd1, char const *file1, int fd2, char const *file2, int flag)
 # endif
 
   if (!flag)
-    return linkat (fd1, file1, fd2, file2, flag);
+    {
+      int result = linkat (fd1, file1, fd2, file2, flag);
+# if LINKAT_SYMLINK_NOTSUP
+      /* OS X 10.10 has linkat() but it doesn't support
+         hardlinks to symlinks.  Fallback to our emulation
+         in that case.  */
+      if (result == -1 && (errno == ENOTSUP || errno == EOPNOTSUPP))
+        return at_func2 (fd1, file1, fd2, file2, link_immediate);
+# endif
+      return result;
+    }
 
   /* Cache the information on whether the system call really works.  */
   {
