@@ -1,5 +1,5 @@
 /* Include and determine availability of smack routines
-   Copyright (C) 2013-2014 Free Software Foundation, Inc.
+   Copyright (C) 2013-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,13 +12,11 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Here we replace or wrap the most common smack functions used by coreutils.
    Others will need to be protected by HAVE_SMACK.  */
 
-#include <config.h>
-
 #ifdef HAVE_SMACK
 # include <sys/smack.h>
 #else
@@ -29,7 +27,7 @@ smack_new_label_from_self (char **label)
 }
 
 static inline int
-smack_set_label_for_self (const char *label)
+smack_set_label_for_self (char const *label)
 {
   return -1;
 }
