@@ -2,13 +2,19 @@ static char const G_line[] =
 {
   '#',' ','C','o','n','f','i','g','u','r','a','t','i','o','n',' ','f','i','l','e',' ','f','o','r',' ','d','i','r','c','o','l','o','r','s',',',' ','a',' ','u','t','i','l','i','t','y',' ','t','o',' ','h','e','l','p',' ','y','o','u',' ','s','e','t',' ','t','h','e',0,
   '#',' ','L','S','_','C','O','L','O','R','S',' ','e','n','v','i','r','o','n','m','e','n','t',' ','v','a','r','i','a','b','l','e',' ','u','s','e','d',' ','b','y',' ','G','N','U',' ','l','s',' ','w','i','t','h',' ','t','h','e',' ','-','-','c','o','l','o','r',' ','o','p','t','i','o','n','.',0,
-  '#',' ','C','o','p','y','r','i','g','h','t',' ','(','C',')',' ','1','9','9','6','-','2','0','2','1',' ','F','r','e','e',' ','S','o','f','t','w','a','r','e',' ','F','o','u','n','d','a','t','i','o','n',',',' ','I','n','c','.',0,
+  '#',' ','C','o','p','y','r','i','g','h','t',' ','(','C',')',' ','1','9','9','6','-','2','0','2','3',' ','F','r','e','e',' ','S','o','f','t','w','a','r','e',' ','F','o','u','n','d','a','t','i','o','n',',',' ','I','n','c','.',0,
   '#',' ','C','o','p','y','i','n','g',' ','a','n','d',' ','d','i','s','t','r','i','b','u','t','i','o','n',' ','o','f',' ','t','h','i','s',' ','f','i','l','e',',',' ','w','i','t','h',' ','o','r',' ','w','i','t','h','o','u','t',' ','m','o','d','i','f','i','c','a','t','i','o','n',',',0,
   '#',' ','a','r','e',' ','p','e','r','m','i','t','t','e','d',' ','p','r','o','v','i','d','e','d',' ','t','h','e',' ','c','o','p','y','r','i','g','h','t',' ','n','o','t','i','c','e',' ','a','n','d',' ','t','h','i','s',' ','n','o','t','i','c','e',' ','a','r','e',' ','p','r','e','s','e','r','v','e','d','.',0,
+  '#',0,
   '#',' ','T','h','e',' ','k','e','y','w','o','r','d','s',' ','C','O','L','O','R',',',' ','O','P','T','I','O','N','S',',',' ','a','n','d',' ','E','I','G','H','T','B','I','T',' ','(','h','o','n','o','r','e','d',' ','b','y',' ','t','h','e',0,
   '#',' ','s','l','a','c','k','w','a','r','e',' ','v','e','r','s','i','o','n',' ','o','f',' ','d','i','r','c','o','l','o','r','s',')',' ','a','r','e',' ','r','e','c','o','g','n','i','z','e','d',' ','b','u','t',' ','i','g','n','o','r','e','d','.',0,
-  '#',' ','B','e','l','o','w',' ','a','r','e',' ','T','E','R','M',' ','e','n','t','r','i','e','s',',',' ','w','h','i','c','h',' ','c','a','n',' ','b','e',' ','a',' ','g','l','o','b',' ','p','a','t','t','e','r','n','s',',',' ','t','o',' ','m','a','t','c','h',0,
-  '#',' ','a','g','a','i','n','s','t',' ','t','h','e',' ','T','E','R','M',' ','e','n','v','i','r','o','n','m','e','n','t',' ','v','a','r','i','a','b','l','e',' ','t','o',' ','d','e','t','e','r','m','i','n','e',' ','i','f',' ','i','t',' ','i','s',' ','c','o','l','o','r','i','z','a','b','l','e','.',0,
+  '#',' ','G','l','o','b','a','l',' ','c','o','n','f','i','g',' ','o','p','t','i','o','n','s',' ','c','a','n',' ','b','e',' ','s','p','e','c','i','f','i','e','d',' ','b','e','f','o','r','e',' ','T','E','R','M',' ','o','r',' ','C','O','L','O','R','T','E','R','M',' ','e','n','t','r','i','e','s',0,
+  '#',' ','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=',0,
+  '#',' ','T','e','r','m','i','n','a','l',' ','f','i','l','t','e','r','s',0,
+  '#',' ','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=',0,
+  '#',' ','B','e','l','o','w',' ','a','r','e',' ','T','E','R','M',' ','o','r',' ','C','O','L','O','R','T','E','R','M',' ','e','n','t','r','i','e','s',',',' ','w','h','i','c','h',' ','c','a','n',' ','b','e',' ','g','l','o','b',' ','p','a','t','t','e','r','n','s',',',' ','w','h','i','c','h',0,
+  '#',' ','r','e','s','t','r','i','c','t',' ','f','o','l','l','o','w','i','n','g',' ','c','o','n','f','i','g',' ','t','o',' ','s','y','s','t','e','m','s',' ','w','i','t','h',' ','m','a','t','c','h','i','n','g',' ','e','n','v','i','r','o','n','m','e','n','t',' ','v','a','r','i','a','b','l','e','s','.',0,
+  'C','O','L','O','R','T','E','R','M',' ','?','*',0,
   'T','E','R','M',' ','E','t','e','r','m',0,
   'T','E','R','M',' ','a','n','s','i',0,
   'T','E','R','M',' ','*','c','o','l','o','r','*',0,
@@ -34,6 +40,9 @@ static char const G_line[] =
   'T','E','R','M',' ','t','m','u','x','*',0,
   'T','E','R','M',' ','v','t','1','0','0',0,
   'T','E','R','M',' ','x','t','e','r','m','*',0,
+  '#',' ','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=',0,
+  '#',' ','B','a','s','i','c',' ','f','i','l','e',' ','a','t','t','r','i','b','u','t','e','s',0,
+  '#',' ','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=',0,
   '#',' ','B','e','l','o','w',' ','a','r','e',' ','t','h','e',' ','c','o','l','o','r',' ','i','n','i','t',' ','s','t','r','i','n','g','s',' ','f','o','r',' ','t','h','e',' ','b','a','s','i','c',' ','f','i','l','e',' ','t','y','p','e','s','.',0,
   '#',' ','O','n','e',' ','c','a','n',' ','u','s','e',' ','c','o','d','e','s',' ','f','o','r',' ','2','5','6',' ','o','r',' ','m','o','r','e',' ','c','o','l','o','r','s',' ','s','u','p','p','o','r','t','e','d',' ','b','y',' ','m','o','d','e','r','n',' ','t','e','r','m','i','n','a','l','s','.',0,
   '#',' ','T','h','e',' ','d','e','f','a','u','l','t',' ','c','o','l','o','r',' ','c','o','d','e','s',' ','u','s','e',' ','t','h','e',' ','c','a','p','a','b','i','l','i','t','i','e','s',' ','o','f',' ','a','n',' ','8',' ','c','o','l','o','r',' ','t','e','r','m','i','n','a','l',0,
@@ -60,26 +69,32 @@ static char const G_line[] =
   'M','I','S','S','I','N','G',' ','0','0',' ','#',' ','.','.','.',' ','a','n','d',' ','t','h','e',' ','f','i','l','e','s',' ','t','h','e','y',' ','p','o','i','n','t',' ','t','o',0,
   'S','E','T','U','I','D',' ','3','7',';','4','1',' ','#',' ','f','i','l','e',' ','t','h','a','t',' ','i','s',' ','s','e','t','u','i','d',' ','(','u','+','s',')',0,
   'S','E','T','G','I','D',' ','3','0',';','4','3',' ','#',' ','f','i','l','e',' ','t','h','a','t',' ','i','s',' ','s','e','t','g','i','d',' ','(','g','+','s',')',0,
-  'C','A','P','A','B','I','L','I','T','Y',' ','3','0',';','4','1',' ','#',' ','f','i','l','e',' ','w','i','t','h',' ','c','a','p','a','b','i','l','i','t','y',0,
+  'C','A','P','A','B','I','L','I','T','Y',' ','0','0',' ','#',' ','f','i','l','e',' ','w','i','t','h',' ','c','a','p','a','b','i','l','i','t','y',' ','(','v','e','r','y',' ','e','x','p','e','n','s','i','v','e',' ','t','o',' ','l','o','o','k','u','p',')',0,
   'S','T','I','C','K','Y','_','O','T','H','E','R','_','W','R','I','T','A','B','L','E',' ','3','0',';','4','2',' ','#',' ','d','i','r',' ','t','h','a','t',' ','i','s',' ','s','t','i','c','k','y',' ','a','n','d',' ','o','t','h','e','r','-','w','r','i','t','a','b','l','e',' ','(','+','t',',','o','+','w',')',0,
   'O','T','H','E','R','_','W','R','I','T','A','B','L','E',' ','3','4',';','4','2',' ','#',' ','d','i','r',' ','t','h','a','t',' ','i','s',' ','o','t','h','e','r','-','w','r','i','t','a','b','l','e',' ','(','o','+','w',')',' ','a','n','d',' ','n','o','t',' ','s','t','i','c','k','y',0,
   'S','T','I','C','K','Y',' ','3','7',';','4','4',' ','#',' ','d','i','r',' ','w','i','t','h',' ','t','h','e',' ','s','t','i','c','k','y',' ','b','i','t',' ','s','e','t',' ','(','+','t',')',' ','a','n','d',' ','n','o','t',' ','o','t','h','e','r','-','w','r','i','t','a','b','l','e',0,
   '#',' ','T','h','i','s',' ','i','s',' ','f','o','r',' ','f','i','l','e','s',' ','w','i','t','h',' ','e','x','e','c','u','t','e',' ','p','e','r','m','i','s','s','i','o','n',':',0,
   'E','X','E','C',' ','0','1',';','3','2',0,
+  '#',' ','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=',0,
+  '#',' ','F','i','l','e',' ','e','x','t','e','n','s','i','o','n',' ','a','t','t','r','i','b','u','t','e','s',0,
+  '#',' ','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=','=',0,
   '#',' ','L','i','s','t',' ','a','n','y',' ','f','i','l','e',' ','e','x','t','e','n','s','i','o','n','s',' ','l','i','k','e',' ','\'','.','g','z','\'',' ','o','r',' ','\'','.','t','a','r','\'',' ','t','h','a','t',' ','y','o','u',' ','w','o','u','l','d',' ','l','i','k','e',' ','l','s',0,
-  '#',' ','t','o',' ','c','o','l','o','r','i','z','e',' ','b','e','l','o','w','.',' ','P','u','t',' ','t','h','e',' ','e','x','t','e','n','s','i','o','n',',',' ','a',' ','s','p','a','c','e',',',' ','a','n','d',' ','t','h','e',' ','c','o','l','o','r',' ','i','n','i','t',' ','s','t','r','i','n','g','.',0,
-  '#',' ','(','a','n','d',' ','a','n','y',' ','c','o','m','m','e','n','t','s',' ','y','o','u',' ','w','a','n','t',' ','t','o',' ','a','d','d',' ','a','f','t','e','r',' ','a',' ','\'','#','\'',')',0,
+  '#',' ','t','o',' ','c','o','l','o','r',' ','b','e','l','o','w','.',' ','P','u','t',' ','t','h','e',' ','s','u','f','f','i','x',',',' ','a',' ','s','p','a','c','e',',',' ','a','n','d',' ','t','h','e',' ','c','o','l','o','r',' ','i','n','i','t',' ','s','t','r','i','n','g','.',0,
+  '#',' ','(','a','n','d',' ','a','n','y',' ','c','o','m','m','e','n','t','s',' ','y','o','u',' ','w','a','n','t',' ','t','o',' ','a','d','d',' ','a','f','t','e','r',' ','a',' ','\'','#','\'',')','.',0,
+  '#',' ','S','u','f','f','i','x','e','s',' ','a','r','e',' ','m','a','t','c','h','e','d',' ','c','a','s','e',' ','i','n','s','e','n','s','i','t','i','v','e','l','y',',',' ','b','u','t',' ','i','f',' ','y','o','u',' ','d','e','f','i','n','e',' ','d','i','f','f','e','r','e','n','t',0,
+  '#',' ','i','n','i','t',' ','s','t','r','i','n','g','s',' ','f','o','r',' ','s','e','p','a','r','a','t','e',' ','c','a','s','e','s',',',' ','t','h','o','s','e',' ','w','i','l','l',' ','b','e',' ','h','o','n','o','r','e','d','.',0,
+  '#',0,
   '#',' ','I','f',' ','y','o','u',' ','u','s','e',' ','D','O','S','-','s','t','y','l','e',' ','s','u','f','f','i','x','e','s',',',' ','y','o','u',' ','m','a','y',' ','w','a','n','t',' ','t','o',' ','u','n','c','o','m','m','e','n','t',' ','t','h','e',' ','f','o','l','l','o','w','i','n','g',':',0,
   '#','.','c','m','d',' ','0','1',';','3','2',' ','#',' ','e','x','e','c','u','t','a','b','l','e','s',' ','(','b','r','i','g','h','t',' ','g','r','e','e','n',')',0,
   '#','.','e','x','e',' ','0','1',';','3','2',0,
   '#','.','c','o','m',' ','0','1',';','3','2',0,
   '#','.','b','t','m',' ','0','1',';','3','2',0,
   '#','.','b','a','t',' ','0','1',';','3','2',0,
-  '#',' ','O','r',' ','i','f',' ','y','o','u',' ','w','a','n','t',' ','t','o',' ','c','o','l','o','r','i','z','e',' ','s','c','r','i','p','t','s',' ','e','v','e','n',' ','i','f',' ','t','h','e','y',' ','d','o',' ','n','o','t',' ','h','a','v','e',' ','t','h','e',0,
+  '#',' ','O','r',' ','i','f',' ','y','o','u',' ','w','a','n','t',' ','t','o',' ','c','o','l','o','r',' ','s','c','r','i','p','t','s',' ','e','v','e','n',' ','i','f',' ','t','h','e','y',' ','d','o',' ','n','o','t',' ','h','a','v','e',' ','t','h','e',0,
   '#',' ','e','x','e','c','u','t','a','b','l','e',' ','b','i','t',' ','a','c','t','u','a','l','l','y',' ','s','e','t','.',0,
   '#','.','s','h',' ','0','1',';','3','2',0,
   '#','.','c','s','h',' ','0','1',';','3','2',0,
-  ' ','#',' ','a','r','c','h','i','v','e','s',' ','o','r',' ','c','o','m','p','r','e','s','s','e','d',' ','(','b','r','i','g','h','t',' ','r','e','d',')',0,
+  '#',' ','a','r','c','h','i','v','e','s',' ','o','r',' ','c','o','m','p','r','e','s','s','e','d',' ','(','b','r','i','g','h','t',' ','r','e','d',')',0,
   '.','t','a','r',' ','0','1',';','3','1',0,
   '.','t','g','z',' ','0','1',';','3','1',0,
   '.','a','r','c',' ','0','1',';','3','1',0,
@@ -127,6 +142,7 @@ static char const G_line[] =
   '.','d','w','m',' ','0','1',';','3','1',0,
   '.','e','s','d',' ','0','1',';','3','1',0,
   '#',' ','i','m','a','g','e',' ','f','o','r','m','a','t','s',0,
+  '.','a','v','i','f',' ','0','1',';','3','5',0,
   '.','j','p','g',' ','0','1',';','3','5',0,
   '.','j','p','e','g',' ','0','1',';','3','5',0,
   '.','m','j','p','g',' ','0','1',';','3','5',0,
@@ -196,4 +212,25 @@ static char const G_line[] =
   '.','o','p','u','s',' ','0','0',';','3','6',0,
   '.','s','p','x',' ','0','0',';','3','6',0,
   '.','x','s','p','f',' ','0','0',';','3','6',0,
+  '#',' ','b','a','c','k','u','p',' ','f','i','l','e','s',0,
+  '*','~',' ','0','0',';','9','0',0,
+  '*','#',' ','0','0',';','9','0',0,
+  '.','b','a','k',' ','0','0',';','9','0',0,
+  '.','o','l','d',' ','0','0',';','9','0',0,
+  '.','o','r','i','g',' ','0','0',';','9','0',0,
+  '.','p','a','r','t',' ','0','0',';','9','0',0,
+  '.','r','e','j',' ','0','0',';','9','0',0,
+  '.','s','w','p',' ','0','0',';','9','0',0,
+  '.','t','m','p',' ','0','0',';','9','0',0,
+  '.','d','p','k','g','-','d','i','s','t',' ','0','0',';','9','0',0,
+  '.','d','p','k','g','-','o','l','d',' ','0','0',';','9','0',0,
+  '.','u','c','f','-','d','i','s','t',' ','0','0',';','9','0',0,
+  '.','u','c','f','-','n','e','w',' ','0','0',';','9','0',0,
+  '.','u','c','f','-','o','l','d',' ','0','0',';','9','0',0,
+  '.','r','p','m','n','e','w',' ','0','0',';','9','0',0,
+  '.','r','p','m','o','r','i','g',' ','0','0',';','9','0',0,
+  '.','r','p','m','s','a','v','e',' ','0','0',';','9','0',0,
+  '#',0,
+  '#',' ','S','u','b','s','e','q','u','e','n','t',' ','T','E','R','M',' ','o','r',' ','C','O','L','O','R','T','E','R','M',' ','e','n','t','r','i','e','s',',',' ','c','a','n',' ','b','e',' ','u','s','e','d',' ','t','o',' ','a','d','d',' ','/',' ','o','v','e','r','r','i','d','e',0,
+  '#',' ','c','o','n','f','i','g',' ','s','p','e','c','i','f','i','c',' ','t','o',' ','t','h','o','s','e',' ','m','a','t','c','h','i','n','g',' ','e','n','v','i','r','o','n','m','e','n','t',' ','v','a','r','i','a','b','l','e','s','.',0,
 };
