@@ -1,5 +1,5 @@
 /* operand2sig.c -- common function for parsing signal specifications
-   Copyright (C) 2008-2011 Free Software Foundation, Inc.
+   Copyright (C) 2008-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Extracted from kill.c/timeout.c by Pádraig Brady.
    FIXME: Move this to gnulib/str2sig.c */
@@ -28,6 +28,7 @@
 
 #include "system.h"
 #include "error.h"
+#include "quote.h"
 #include "sig2str.h"
 #include "operand2sig.h"
 
@@ -38,11 +39,29 @@ operand2sig (char const *operand, char *signame)
 
   if (ISDIGIT (*operand))
     {
+      /* Note we don't put a limit on the maximum value passed,
+         because we're checking shell $? values here, and ksh for
+         example will add 256 to the signal value, thus being wider
+         than the number of WEXITSTATUS bits.
+         We could validate that values were not above say
+         ((WEXITSTATUS (~0) << 1) + 1), which would cater for ksh.
+         But some shells may use other adjustments in future to be
+         (forward) compatible with systems that support
+         wider exit status values as discussed at
+         https://austingroupbugs.net/view.php?id=947  */
+
       char *endp;
       long int l = (errno = 0, strtol (operand, &endp, 10));
       int i = l;
-      signum = (operand == endp || *endp || errno || i != l ? -1
-                : WIFSIGNALED (i) ? WTERMSIG (i) : i);
+      signum = (operand == endp || *endp || errno || i != l ? -1 : i);
+
+      if (signum != -1)
+        {
+          /* Note AIX uses a different bit pattern for status returned
+             from shell and wait(), so we can't use WTERMSIG etc. here.
+             Also ksh returns 0xFF + signal number.  */
+          signum &= signum >= 0xFF ? 0xFF : 0x7F;
+        }
     }
   else
     {
@@ -66,7 +85,7 @@ operand2sig (char const *operand, char *signame)
 
   if (signum < 0 || sig2str (signum, signame) != 0)
     {
-      error (0, 0, _("%s: invalid signal"), operand);
+      error (0, 0, _("%s: invalid signal"), quote (operand));
       return -1;
     }
 
