@@ -1,5 +1,5 @@
 /* printf - format and print data
-   Copyright (C) 1990-2010 Free Software Foundation, Inc.
+   Copyright (C) 1990-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Usage: printf format [argument...]
 
@@ -41,7 +41,11 @@
    %b = print an argument string, interpreting backslash escapes,
      except that octal escapes are of the form \0 or \0ooo.
 
-   The `format' argument is re-used as many times as necessary
+   %q = print an argument string in a format that can be
+     reused as shell input.  Escaped characters used the proposed
+     POSIX $'' syntax supported by most shells.
+
+   The 'format' argument is re-used as many times as necessary
    to convert all of the given arguments.
 
    David MacKenzie <djm@gnu.ai.mit.edu> */
@@ -49,15 +53,17 @@
 #include <config.h>
 #include <stdio.h>
 #include <sys/types.h>
+#include <wchar.h>
 
 #include "system.h"
-#include "c-strtod.h"
+#include "cl-strtod.h"
+#include "die.h"
 #include "error.h"
 #include "quote.h"
 #include "unicodeio.h"
 #include "xprintf.h"
 
-/* The official name of this program (e.g., no `g' prefix).  */
+/* The official name of this program (e.g., no 'g' prefix).  */
 #define PROGRAM_NAME "printf"
 
 #define AUTHORS proper_name ("David MacKenzie")
@@ -82,8 +88,7 @@ void
 usage (int status)
 {
   if (status != EXIT_SUCCESS)
-    fprintf (stderr, _("Try `%s --help' for more information.\n"),
-             program_name);
+    emit_try_help ();
   else
     {
       printf (_("\
@@ -123,32 +128,34 @@ FORMAT controls the output as in C printf.  Interpreted sequences are:\n\
 "), stdout);
       fputs (_("\
   %%      a single %\n\
-  %b      ARGUMENT as a string with `\\' escapes interpreted,\n\
+  %b      ARGUMENT as a string with '\\' escapes interpreted,\n\
           except that octal escapes are of the form \\0 or \\0NNN\n\
-\n\
+  %q      ARGUMENT is printed in a format that can be reused as shell input,\n\
+          escaping non-printable characters with the proposed POSIX $'' syntax.\
+\n\n\
 and all C format specifications ending with one of diouxXfeEgGcs, with\n\
 ARGUMENTs converted to proper type first.  Variable widths are handled.\n\
 "), stdout);
       printf (USAGE_BUILTIN_WARNING, PROGRAM_NAME);
-      emit_ancillary_info ();
+      emit_ancillary_info (PROGRAM_NAME);
     }
   exit (status);
 }
 
 static void
-verify_numeric (const char *s, const char *end)
+verify_numeric (char const *s, char const *end)
 {
   if (errno)
     {
-      error (0, errno, "%s", s);
+      error (0, errno, "%s", quote (s));
       exit_status = EXIT_FAILURE;
     }
   else if (*end)
     {
       if (s == end)
-        error (0, 0, _("%s: expected a numeric value"), s);
+        error (0, 0, _("%s: expected a numeric value"), quote (s));
       else
-        error (0, 0, _("%s: value not completely converted"), s);
+        error (0, 0, _("%s: value not completely converted"), quote (s));
       exit_status = EXIT_FAILURE;
     }
 }
@@ -160,10 +167,25 @@ FUNC_NAME (char const *s)						 \
   char *end;								 \
   TYPE val;								 \
                                                                          \
-  if (*s == '\"' || *s == '\'')						 \
+  if ((*s == '\"' || *s == '\'') && *(s + 1))				 \
     {									 \
       unsigned char ch = *++s;						 \
       val = ch;								 \
+                                                                         \
+      if (MB_CUR_MAX > 1 && *(s + 1))					 \
+        {								 \
+          mbstate_t mbstate = { 0, };					 \
+          wchar_t wc;							 \
+          size_t slen = strlen (s);					 \
+          ssize_t bytes;						 \
+          bytes = mbrtowc (&wc, s, slen, &mbstate);			 \
+          if (0 < bytes)						 \
+            {								 \
+              val = wc;							 \
+              s += bytes - 1;						 \
+            }								 \
+        }								 \
+                                                                         \
       /* If POSIXLY_CORRECT is not set, then give a warning that there	 \
          are characters following the character constant and that GNU	 \
          printf is ignoring those characters.  If POSIXLY_CORRECT *is*	 \
@@ -182,7 +204,7 @@ FUNC_NAME (char const *s)						 \
 
 STRTOX (intmax_t,    vstrtoimax, strtoimax (s, &end, 0))
 STRTOX (uintmax_t,   vstrtoumax, strtoumax (s, &end, 0))
-STRTOX (long double, vstrtold,   c_strtold (s, &end))
+STRTOX (long double, vstrtold,   cl_strtold (s, &end))
 
 /* Output a single-character \ escape.  */
 
@@ -231,9 +253,9 @@ print_esc_char (char c)
    is an octal digit; otherwise they are of the form \ooo.  */
 
 static int
-print_esc (const char *escstart, bool octal_0)
+print_esc (char const *escstart, bool octal_0)
 {
-  const char *p = escstart + 1;
+  char const *p = escstart + 1;
   int esc_value = 0;		/* Value of \nnn escape. */
   int esc_length;		/* Length of \nnn escape. */
 
@@ -245,7 +267,7 @@ print_esc (const char *escstart, bool octal_0)
            ++esc_length, ++p)
         esc_value = esc_value * 16 + hextobin (*p);
       if (esc_length == 0)
-        error (EXIT_FAILURE, 0, _("missing hexadecimal number in escape"));
+        die (EXIT_FAILURE, 0, _("missing hexadecimal number in escape"));
       putchar (esc_value);
     }
   else if (isodigit (*p))
@@ -272,20 +294,15 @@ print_esc (const char *escstart, bool octal_0)
            --esc_length, ++p)
         {
           if (! isxdigit (to_uchar (*p)))
-            error (EXIT_FAILURE, 0, _("missing hexadecimal number in escape"));
+            die (EXIT_FAILURE, 0, _("missing hexadecimal number in escape"));
           uni_value = uni_value * 16 + hextobin (*p);
         }
 
-      /* A universal character name shall not specify a character short
-         identifier in the range 00000000 through 00000020, 0000007F through
-         0000009F, or 0000D800 through 0000DFFF inclusive. A universal
-         character name shall not designate a character in the required
-         character set.  */
-      if ((uni_value <= 0x9f
-           && uni_value != 0x24 && uni_value != 0x40 && uni_value != 0x60)
-          || (uni_value >= 0xd800 && uni_value <= 0xdfff))
-        error (EXIT_FAILURE, 0, _("invalid universal character name \\%c%0*x"),
-               esc_char, (esc_char == 'u' ? 4 : 8), uni_value);
+      /* Error for invalid code points 0000D800 through 0000DFFF inclusive.
+         Note print_unicode_char() would print the literal \u.. in this case. */
+      if (uni_value >= 0xd800 && uni_value <= 0xdfff)
+        die (EXIT_FAILURE, 0, _("invalid universal character name \\%c%0*x"),
+             esc_char, (esc_char == 'u' ? 4 : 8), uni_value);
 
       print_unicode_char (stdout, uni_value, 0);
     }
@@ -304,7 +321,7 @@ print_esc (const char *escstart, bool octal_0)
 /* Print string STR, evaluating \ escapes. */
 
 static void
-print_esc_string (const char *str)
+print_esc_string (char const *str)
 {
   for (; *str; str++)
     if (*str == '\\')
@@ -322,7 +339,7 @@ print_esc_string (const char *str)
    be formatted.  */
 
 static void
-print_direc (const char *start, size_t length, char conversion,
+print_direc (char const *start, size_t length, char conversion,
              bool have_field_width, int field_width,
              bool have_precision, int precision,
              char const *argument)
@@ -465,15 +482,15 @@ print_direc (const char *start, size_t length, char conversion,
 }
 
 /* Print the text in FORMAT, using ARGV (with ARGC elements) for
-   arguments to any `%' directives.
+   arguments to any '%' directives.
    Return the number of elements of ARGV used.  */
 
 static int
-print_formatted (const char *format, int argc, char **argv)
+print_formatted (char const *format, int argc, char **argv)
 {
   int save_argc = argc;		/* Preserve original value.  */
-  const char *f;		/* Pointer into `format'.  */
-  const char *direc_start;	/* Start of % directive.  */
+  char const *f;		/* Pointer into 'format'.  */
+  char const *direc_start;	/* Start of % directive.  */
   size_t direc_length;		/* Length of % directive.  */
   bool have_field_width;	/* True if FIELD_WIDTH is valid.  */
   int field_width = 0;		/* Arg to first '*'.  */
@@ -507,6 +524,18 @@ print_formatted (const char *format, int argc, char **argv)
               break;
             }
 
+          if (*f == 'q')
+            {
+              if (argc > 0)
+                {
+                  fputs (quotearg_style (shell_escape_quoting_style, *argv),
+                         stdout);
+                  ++argv;
+                  --argc;
+                }
+              break;
+            }
+
           memset (ok, 0, sizeof ok);
           ok['a'] = ok['A'] = ok['c'] = ok['d'] = ok['e'] = ok['E'] =
             ok['f'] = ok['F'] = ok['g'] = ok['G'] = ok['i'] = ok['o'] =
@@ -533,7 +562,7 @@ print_formatted (const char *format, int argc, char **argv)
               default:
                 goto no_more_flag_characters;
               }
-        no_more_flag_characters:;
+        no_more_flag_characters:
 
           if (*f == '*')
             {
@@ -545,8 +574,8 @@ print_formatted (const char *format, int argc, char **argv)
                   if (INT_MIN <= width && width <= INT_MAX)
                     field_width = width;
                   else
-                    error (EXIT_FAILURE, 0, _("invalid field width: %s"),
-                           *argv);
+                    die (EXIT_FAILURE, 0, _("invalid field width: %s"),
+                         quote (*argv));
                   ++argv;
                   --argc;
                 }
@@ -580,8 +609,8 @@ print_formatted (const char *format, int argc, char **argv)
                           precision = -1;
                         }
                       else if (INT_MAX < prec)
-                        error (EXIT_FAILURE, 0, _("invalid precision: %s"),
-                               *argv);
+                        die (EXIT_FAILURE, 0, _("invalid precision: %s"),
+                             quote (*argv));
                       else
                         precision = prec;
                       ++argv;
@@ -606,9 +635,9 @@ print_formatted (const char *format, int argc, char **argv)
           {
             unsigned char conversion = *f;
             if (! ok[conversion])
-              error (EXIT_FAILURE, 0,
-                     _("%.*s: invalid conversion specification"),
-                     (int) (f + 1 - direc_start), direc_start);
+              die (EXIT_FAILURE, 0,
+                   _("%.*s: invalid conversion specification"),
+                   (int) (f + 1 - direc_start), direc_start);
           }
 
           print_direc (direc_start, direc_length, *f,
@@ -658,12 +687,12 @@ main (int argc, char **argv)
         {
           version_etc (stdout, PROGRAM_NAME, PACKAGE_NAME, Version, AUTHORS,
                        (char *) NULL);
-          exit (EXIT_SUCCESS);
+          return EXIT_SUCCESS;
         }
     }
 
   /* The above handles --help and --version.
-     Since there is no other invocation of getopt, handle `--' here.  */
+     Since there is no other invocation of getopt, handle '--' here.  */
   if (1 < argc && STREQ (argv[1], "--"))
     {
       --argc;
@@ -693,5 +722,5 @@ main (int argc, char **argv)
            _("warning: ignoring excess arguments, starting with %s"),
            quote (argv[0]));
 
-  exit (exit_status);
+  return exit_status;
 }
