@@ -1,6 +1,6 @@
 /* tac from a pipe.
 
-   Copyright (C) 1997-2013 Free Software Foundation, Inc.
+   Copyright (C) 1997-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,11 +13,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* FIXME */
 #include <assert.h>
 
+#include "die.h"
+
 /* FIXME: this is small for testing */
 #define BUFFER_SIZE (8)
 
@@ -56,7 +58,7 @@ buf_init_from_stdin (Buf *x, char eol_byte)
 #define OBS (&(x->obs))
   obstack_init (OBS);
 
-  while (1)
+  while (true)
     {
       char *buf = (char *) malloc (BUFFER_SIZE);
       size_t bytes_read;
@@ -71,7 +73,7 @@ buf_init_from_stdin (Buf *x, char eol_byte)
         }
       bytes_read = full_read (STDIN_FILENO, buf, BUFFER_SIZE);
       if (bytes_read != buffer_size && errno != 0)
-        error (EXIT_FAILURE, errno, _("read error"));
+        die (EXIT_FAILURE, errno, _("read error"));
 
       {
         struct B_pair bp;
@@ -125,8 +127,7 @@ buf_init_from_stdin (Buf *x, char eol_byte)
 static void
 buf_free (Buf *x)
 {
-  size_t i;
-  for (i = 0; i < x->n_bufs; i++)
+  for (size_t i = 0; i < x->n_bufs; i++)
     free (x->p[i].start);
   obstack_free (OBS, NULL);
 }
@@ -184,7 +185,7 @@ find_bol (const Buf *x,
   tmp = line_ptr_decrement (x, last_bol);
   last_bol_ptr = tmp.ptr;
   i = tmp.i;
-  while (1)
+  while (true)
     {
       char *nl = memrchr (x->p[i].start, last_bol_ptr, eol_byte);
       if (nl)
@@ -220,8 +221,7 @@ static void
 print_line (FILE *out_stream, const Buf *x,
             const Line_ptr *bol, const Line_ptr *bol_next)
 {
-  size_t i;
-  for (i = bol->i; i <= bol_next->i; i++)
+  for (size_t i = bol->i; i <= bol_next->i; i++)
     {
       char *a = (i == bol->i ? bol->ptr : x->p[i].start);
       char *b = (i == bol_next->i ? bol_next->ptr : ONE_PAST_END (x, i));
@@ -250,7 +250,7 @@ tac_mem ()
   bol.i = x.n_bufs - 1;
   bol.ptr = ONE_PAST_END (&x, bol.i);
 
-  while (1)
+  while (true)
     {
       Line_ptr new_bol;
       if (! find_bol (&x, &bol, &new_bol, eol_byte))
