@@ -1,7 +1,7 @@
 #!/bin/sh
 # exercise another small part of remove.c
 
-# Copyright (C) 2002-2012 Free Software Foundation, Inc.
+# Copyright (C) 2002-2023 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,14 +14,14 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 . "${srcdir=.}/tests/init.sh"; path_prepend_ ./src
 print_ver_ rm
 skip_if_root_
 
 mkdir -p b/a/p b/c b/d || framework_failure_
-chmod u-w b/a || framework_failure_
+chmod ug-w b/a || framework_failure_
 
 
 # This should fail.
